@@ -29,9 +29,14 @@
 #include <time.h>
 #include <sys/time.h>
 #include <unistd.h>
+#include <locale.h>
 
 #ifdef _WIN32
 #include <windows.h>
+#include <direct.h>
+#else
+#include <sys/stat.h>
+#include <sys/types.h>
 #endif
 
 #ifdef WITH_SIGNATURE // gpg signed verified binary
@@ -88,6 +93,7 @@ typedef struct {
   short update_gui_now;
   short update_pgm_now;
   short swap_instances;
+  short queue_panic;
 
   unsigned int active_keys [MAX_KEYS/32];
 
@@ -372,6 +378,8 @@ save(LV2_Handle                instance,
 {
   B3S* b3s = (B3S*)instance;
 
+  LOCALEGUARD_START;
+
   char *cfg = (char*) calloc(1, sizeof(char));
   rc_loop_state(b3s->inst->state, rcstate_cb, (void*) &cfg);
 
@@ -415,6 +423,8 @@ save(LV2_Handle                instance,
   cfg = (char*) realloc(cfg, strlen(cfg) + strlen(out) +1);
   strcat(cfg, out);
 
+  LOCALEGUARD_END;
+
   store(handle, b3s->uris.sb3_state,
       cfg, strlen(cfg) + 1,
       b3s->uris.atom_String,
@@ -453,6 +463,8 @@ restore(LV2_Handle                  instance,
   const char* cfg = (const char*)value;
   const char *te, *ts = cfg;
 
+  LOCALEGUARD_START;
+
   /* pass1 - evaulate CFG -- before initializing synth */
   while (ts && *ts && (te=strchr(ts, '\n'))) {
     char *val;
@@ -500,6 +512,8 @@ restore(LV2_Handle                  instance,
     ts=te+1;
   }
 
+  LOCALEGUARD_END;
+
   b3s->swap_instances = 1;
   return LV2_STATE_SUCCESS;
 }
@@ -528,6 +542,40 @@ static void clone_cb_mcc(int fnid, const char *key, const char *kv, unsigned cha
   }
 }
 
+#ifdef _WIN32
+#define DIRSEP_C '\\'
+#else
+#define DIRSEP_C '/'
+#endif
+
+
+static void create_containing_dir (const char *path) {
+  size_t len = strlen(path);
+  if (!path || len == 0) return;
+  if(path[len - 1] == DIRSEP_C) {
+    return;
+  }
+
+  char *tmp = strdup(path);
+  char *p = tmp + 1;
+#ifdef _WIN32
+  if (len > 3 && tmp[1] == ':' && tmp[2] == '\\') {
+    p = tmp + 3;
+  }
+#endif
+  for(; *p; ++p)
+    if(*p == DIRSEP_C) {
+      *p = 0;
+#ifdef _WIN32
+      _mkdir(tmp);
+#else
+      mkdir(tmp, 0755);
+#endif
+      *p = DIRSEP_C;
+    }
+  free (tmp);
+}
+
 /* LV2 -- worker */
 static LV2_Worker_Status
 work(LV2_Handle                  instance,
@@ -586,6 +634,7 @@ work(LV2_Handle                  instance,
 	// fprintf(stderr, "B3LV2: adding cfg line: %s\n", w->msg);
 	b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
 	allocSynth(b3s->inst_offline);
+	LOCALEGUARD_START;
 	// clone current state...
 	rc_loop_state(b3s->inst->state, clone_cb_cfg, b3s->inst_offline);
 	// copy program info
@@ -595,6 +644,7 @@ work(LV2_Handle                  instance,
 	initSynth(b3s->inst_offline, SampleRateD);
 	// replay CCs after synth init
 	rc_loop_state(b3s->inst->state, clone_cb_mcc, b3s->inst_offline);
+	LOCALEGUARD_END;
 	w->status = 0;
       }
       break;
@@ -612,14 +662,18 @@ work(LV2_Handle                  instance,
       fprintf(stderr, "B3LV2: loading cfg file: %s\n", w->msg);
       b3s->inst_offline = (b_instance*) calloc(1, sizeof(struct b_instance));
       allocSynth(b3s->inst_offline);
+
       w->status = parseConfigurationFile (b3s->inst_offline, w->msg);
       initSynth(b3s->inst_offline, SampleRateD);
       break;
     case CMD_SAVECFG:
+      create_containing_dir (w->msg);
       x = fopen(w->msg, "w");
       if (x) {
 	fprintf(x, "# setBfree config file\n# modificaions on top of default config\n");
+	LOCALEGUARD_START;
 	rc_loop_state(b3s->inst->state, rcsave_cb, (void*) x);
+	LOCALEGUARD_END;
 	fclose(x);
 	w->status = 0;
       } else {
@@ -627,6 +681,7 @@ work(LV2_Handle                  instance,
       }
       break;
     case CMD_SAVEPGM:
+      create_containing_dir (w->msg);
       x = fopen(w->msg, "w");
       if (x) {
 	fprintf(x, "# setBfree midi program file\n");
@@ -838,6 +893,7 @@ instantiate(const LV2_Descriptor*     descriptor,
   b3s->swap_instances = 0;
   b3s->update_gui_now = 0;
   b3s->update_pgm_now = 0;
+  b3s->queue_panic = 0;
 
   b3s->inst = (b_instance*) calloc(1, sizeof(struct b_instance));
   b3s->inst_offline = NULL;
@@ -850,15 +906,13 @@ instantiate(const LV2_Descriptor*     descriptor,
   char * defaultProgrammeFile = NULL;
 
 #ifdef _WIN32
-  // out of luck with CSIDL_LOCAL_APPDATA
-  const char * homedrive = getenv("HOMEDRIVE");
-  const char * homepath = getenv("HOMEPATH");
-  if (homedrive && homepath && (strlen(homedrive) + strlen(homepath) + 38) < 1024) {
-    size_t hl = strlen(homedrive) + strlen(homepath);
-    defaultConfigFile=(char*) malloc(hl+38);
-    defaultProgrammeFile=(char*) malloc(hl+38);
-    sprintf(defaultConfigFile,    "%s%s\\Local Settings\\setBfree\\default.cfg", homedrive, homepath);
-    sprintf(defaultProgrammeFile, "%s%s\\Local Settings\\setBfree\\default.pgm", homedrive, homepath);
+  char wintmp[1024] = "";
+  if (ExpandEnvironmentStrings("%localappdata%\\setBfree\\default.cfg", wintmp, 1024)) {
+    defaultConfigFile = strdup (wintmp);
+  }
+  wintmp[0] = '\0';
+  if (ExpandEnvironmentStrings("%localappdata%\\setBfree\\default.pgm", wintmp, 1024)) {
+    defaultProgrammeFile = strdup (wintmp);
   }
 #else // unices: prefer XDG_CONFIG_HOME
   if (getenv("XDG_CONFIG_HOME")) {
@@ -987,6 +1041,13 @@ connect_port(LV2_Handle instance,
   }
 }
 
+static void
+activate(LV2_Handle instance)
+{
+  B3S* b3s = (B3S*)instance;
+  b3s->queue_panic = 1;
+}
+
 static void
 run(LV2_Handle instance, uint32_t n_samples)
 {
@@ -1003,7 +1064,7 @@ run(LV2_Handle instance, uint32_t n_samples)
   if (!warning_printed && capacity < 4096) {
     warning_printed = true;
     fprintf(stderr, "B3LV2: LV message buffer is only %d bytes. Expect problems.\n", capacity);
-    fprintf(stderr, "B3LV2: if your LV2 host allows to configure a buffersize use at least 4kBytes.\n");
+    fprintf(stderr, "B3LV2: if your LV2 host allows one to configure a buffersize use at least 4kBytes.\n");
 
   }
   lv2_atom_forge_set_buffer(&b3s->forge, (uint8_t*)b3s->midiout, capacity);
@@ -1011,6 +1072,11 @@ run(LV2_Handle instance, uint32_t n_samples)
 
   uint32_t written = 0;
 
+  if (b3s->queue_panic) {
+	  b3s->queue_panic = 0;
+	  midi_panic(b3s->inst);
+  }
+
   /* Process incoming events from GUI and handle MIDI events */
   if (b3s->midiin) {
     LV2_Atom_Event* ev = lv2_atom_sequence_begin(&(b3s->midiin)->body);
@@ -1145,7 +1211,7 @@ static const LV2_Descriptor descriptor = {
   SB3_URI,
   instantiate,
   connect_port,
-  NULL,
+  activate,
   run,
   NULL,
   cleanup,
